
{% if not warrior.config_manager.all_valid() %}
  <li>
    <p>
      The warrior must be configured before it can run.
      Please answer the questions below.
    </p>
  </li>
{% end %}

{% for config_value in warrior.config_manager.editable_values() %}
  {% set value = posted_values[config_value.name] if posted_values and config_value.name in posted_values else config_value.value %}

  <li class="{% if config_value.error %} error {% elif not config_value.is_valid() %} required {% end %} {% if config_value.advanced %} advanced {% end %}">
    <p><label for="f-settings-{{ escape(str(id(config_value))) }}">{{ escape(config_value.title) }}:</label></p>
    <p>
      <input type="text" value="{{ escape(str(value or "")) }}" id="f-settings-{{ escape(str(id(config_value))) }}" name="{{ escape(config_value.name) }}" size="30" />
    </p>
    {% if config_value.error %}
      <p class="error">{{ escape(config_value.error) }}</p>
    {% end %}
    {% if config_value.description %}
      <p class="info">{{ escape(config_value.description) }}</p>
    {% end %}
  </li>

{% end %}

