{% import sys %}
{% import seesaw %}

<h3>Help</h3>
<ul>
	<li><a href="http://archiveteam.org/index.php?title=Warrior">Warrior instructions and FAQ</a></li>
	<li><a href="https://github.com/ArchiveTeam/seesaw-kit/issues">Issues and bug reports</a></li>
</ul>

<h3>About</h3>
<ul>
	<li><a href="http://archiveteam.org/">ArchiveTeam homepage</a></li>
	<li>Seesaw version: {{ seesaw.__version__ }}</li>
	<li>Python version: {{ sys.version }}</li>
</ul>